from typing import List, Literal, Optional, Union
from enum import Enum

from pydantic import ConfigDict, BaseModel


class QuestionnaireName(str, Enum):
    REGISTRATION = "Registration"
    ACCT_UPGRADE = "Account Upgrade"

    def __str__(self):
        return str(self.value)


class UseReason(str, Enum):
    """
    # TECH_USE = "Technology Development"
    # SCI_USE = "Scientific Research"
    # EDU_USE = "Education"
    # CUR_USE = "Curious"
    # OTH_USE = "Other" """

    SCI_USE = "SCI_USE"
    TECH_USE = "TECH_USE"
    EDU_USE = "EDU_USE"
    CUR_USE = "CUR_USE"
    OTH_USE = "OTH_USE"

    def __str__(self):
        return str(self.value)


class SchoolLevel(str, Enum):
    """
    # PRIMARY = "Primary (K-8)"
    # SECONDARY = "Secondary (9-12)"
    # COLLEGE = "College/University"
    # POSTGRAD = "Post-Graduate"
    # NOTGIVEN = "Not Given"
    PRIMARY = "PRIMARY" """

    SECONDARY = "SECONDARY"
    COLLEGE = "COLLEGE"
    POSTGRAD = "POSTGRAD"
    NOTGIVEN = "NOTGIVEN"

    def __str__(self):
        return str(self.value)


class RegistrationQandA(BaseModel):
    uses: List[UseReason]
    school_level: Optional[SchoolLevel] = SchoolLevel.NOTGIVEN
    model_config = ConfigDict(validate_assignment=True, use_enum_values=True)


class AcctUpgradeQandA(BaseModel):
    uses: List[UseReason]
    more: Optional[str] = None
    model_config = ConfigDict(validate_assignment=True, use_enum_values=True)


class Questionnaire(BaseModel):
    name: QuestionnaireName
    q_and_a: Union[RegistrationQandA, AcctUpgradeQandA]
    model_config = ConfigDict(validate_assignment=True, use_enum_values=True)


class RegistrationQuestionnaire(Questionnaire):
    name: Literal[QuestionnaireName.REGISTRATION] = QuestionnaireName.REGISTRATION
    q_and_a: RegistrationQandA


class AcctUpgradeQuestionnaire(Questionnaire):
    name: Literal[QuestionnaireName.ACCT_UPGRADE] = QuestionnaireName.ACCT_UPGRADE
    q_and_a: AcctUpgradeQandA



from collections import namedtuple

import pytest

from .job import *


@pytest.fixture
def job_fixtures(
    post_bec_job,
    post_barrier_job,
    post_paint1d_job,
    pending_bec_job,
    running_bec_job,
    complete_bec_job,
    complete_barrier_job,
    complete_paint_1d_job,
    failed_bec_job,
    post_bec_batch_job,
    pending_bec_batch_job,
    running_bec_batch_job,
    complete_bec_batch_job,
    failed_bec_batch_job,
):
    JobFixtures = namedtuple(
        "JobFixtures",
        [
            "post_bec_job",
            "post_barrier_job",
            "post_paint1d_job",
            "pending_bec_job",
            "running_bec_job",
            "complete_bec_job",
            "complete_barrier_job",
            "complete_paint_1d_job",
            "failed_bec_job",
            "post_bec_batch_job",
            "pending_bec_batch_job",
            "running_bec_batch_job",
            "complete_bec_batch_job",
            "failed_bec_batch_job",
        ],
    )
    yield JobFixtures(
        post_bec_job,
        post_barrier_job,
        post_paint1d_job,
        pending_bec_job,
        running_bec_job,
        complete_bec_job,
        complete_barrier_job,
        complete_paint_1d_job,
        failed_bec_job,
        post_bec_batch_job,
        pending_bec_batch_job,
        running_bec_batch_job,
        complete_bec_batch_job,
        failed_bec_batch_job,
    )



# type: ignore
from datetime import datetime
from enum import Enum
from typing import Annotated

from pydantic import (
    BaseModel,
    ConfigDict,
    EmailStr,
    HttpUrl,
    StringConstraints,
    model_serializer,
)

from .questionnaire import RegistrationQandA

external_user_id: type[str] = Annotated[
    str, StringConstraints(pattern=r"^auth0|[a-z0-9]{24}$")
]


class RoleName(str, Enum):
    SUPERUSER = "SUPERUSER"
    ORG_ADMIN = "ORG_ADMIN"

    def __str__(self):
        return str(self.value)


class PurchaseEntity(str, Enum):
    USER = "USER"
    PAID = "PAID"
    ORG = "ORG"


class JobLimitType(str, Enum):
    JOB_RATE = "JOB_RATE"
    JOB_QUOTA = "JOB_QUOTA"


class ExternalUserId(BaseModel):
    external_user_id: external_user_id


class RoleBase(BaseModel):
    name: RoleName
    description: str | None = None
    model_config = ConfigDict(from_attributes=True)


class Role(RoleBase):
    pass


class OrganizationBase(BaseModel):
    name: str
    model_config = ConfigDict(from_attributes=True)


class Organization(OrganizationBase):
    id: int
    model_config = ConfigDict(from_attributes=True)


class UserBase(BaseModel):
    name: Annotated[str, StringConstraints(min_length=1, max_length=100)]
    email: EmailStr
    affiliation: str
    model_config = ConfigDict(from_attributes=True, extra="forbid")


class OrganizationUser(UserBase):
    external_user_id: external_user_id
    jobs_run_today: int


class EmailPreference(BaseModel):
    jobs: bool
    general: bool
    model_config = ConfigDict(from_attributes=True)


class UserUpdate(UserBase):
    name: Annotated[str, StringConstraints(min_length=1, max_length=100)] | None = None
    email: EmailStr | None = None
    affiliation: str | None = None
    onboard_date: datetime | None = None
    email_preferences: EmailPreference | None = None


class UserResponse(UserBase):
    signup_date: datetime
    onboard_date: datetime | None = None
    external_user_id: external_user_id
    roles: list[Role] | None = None
    organization: Organization | None = None
    active: bool
    email_preferences: EmailPreference


class UserSearchResponse(UserBase):
    external_user_id: str
    organization: Organization | None = None


class User(UserBase):
    id: int
    external_user_id: external_user_id
    signup_date: datetime
    roles: list[Role] | None = None
    organization: Organization | None = None


class UserCreate(UserBase):
    external_user_id: external_user_id


class UserSignUp(UserBase):
    questionnaire: RegistrationQandA
    response: str | None = None
    model_config = ConfigDict(use_enum_values=True)


class JobLimit(BaseModel):
    daily_limit: int
    daily_used: int
    daily_remaining: int
    purchased: int | None = None
    purchased_remaining: int | None = None


class JobPurchase(BaseModel):
    purchase_date: datetime
    quantity: int
    model_config = ConfigDict(from_attributes=True)


class UserJobPurchase(JobPurchase):
    user_id: int


class OrgJobPurchase(JobPurchase):
    org_id: int


class PurchasePriceData(BaseModel):
    currency: str
    product_data: dict
    unit_amount: int


class PurchaseLineItemQuantity(BaseModel):
    enabled: bool
    minimum: int
    maximum: int


class PurchaseLineItem(BaseModel):
    amount: int | None = None
    currency: str | None = None
    description: str | None = None
    price: str
    quantity: int
    adjustable_quantity: PurchaseLineItemQuantity | None = None
    price_data: PurchasePriceData | None = None


class CheckoutSessionLineItem(BaseModel):
    quantity: int


class CheckoutSession(BaseModel):
    status: str
    created: datetime
    amount_total: int
    line_items: list[CheckoutSessionLineItem]


class PurchaseCharge(BaseModel):
    created: datetime
    amount_captured: int
    currency: str
    card_last4: int
    card_brand: str
    receipt_url: str
    quantity: int


class CheckoutSessionLineItem(BaseModel):
    quantity: int


class CheckoutSessionResponse(BaseModel):
    session_url: HttpUrl


class EmailBase(BaseModel):
    subject: str
    email: EmailStr
    content: Annotated[str, StringConstraints(min_length=1, max_length=500)]


class FeedbackEmail(EmailBase):
    follow_up_requested: bool


class PasswordReset(BaseModel):
    redirect_url: str


class SalesforceLead(BaseModel):
    first_name: Annotated[str, StringConstraints(min_length=1, max_length=40)]
    last_name: Annotated[str, StringConstraints(min_length=1, max_length=80)]
    email: EmailStr
    company: Annotated[str, StringConstraints(min_length=1, max_length=255)]
    external_user_id: external_user_id
    registration_date: datetime

    @model_serializer
    def model(self) -> dict:
        return {
            "FirstName": self.first_name,
            "LastName": self.last_name,
            "Email": self.email,
            "Company": self.company,
            "QMS_User_ID__c": self.external_user_id,
            "QMS_Registration_Date__c": self.registration_date.strftime("%Y-%m-%d"),
        }



from datetime import time
from enum import Enum
from typing import Optional

from pydantic import BaseModel, ConfigDict, field_validator


class QPUName(str, Enum):
    SMALLBERT = "SMALLBERT"
    BIGBERT = "BIGBERT"
    UNDEFINED = "UNDEFINED"
    SIMULATOR = "SIMULATOR"

    def __str__(self):
        return str(self.value)


class QPUStatus(str, Enum):
    ONLINE = "ONLINE"
    OFFLINE = "OFFLINE"

    def __str__(self):
        return str(self.value)


class QPUJobType(BaseModel):
    name: str
    model_config = ConfigDict(from_attributes=True)


class QPUAccess(BaseModel):
    day: str
    start_time: time
    end_time: time
    model_config = ConfigDict(from_attributes=True)


class QPUBase(BaseModel):
    name: QPUName
    status: QPUStatus
    model_config = ConfigDict(from_attributes=True)


class QPU(QPUBase):
    qpu_access: list[QPUAccess]
    job_types: list[QPUJobType]

    @field_validator("job_types")
    @classmethod
    def job_type_names(cls, value):
        return [i.name for i in value]

    model_config = ConfigDict(from_attributes=True)


class QPUState(QPU):
    pending_internal_jobs: int | None = None
    pending_external_jobs: int | None = None


class QPUStatusUpdate(BaseModel):
    status: QPUStatus

    # @root_validator()
    # def check_status_or_hours(cls, values):
    #     if (values.get("status") is None) and (values.get("operation_hours") is None):
    #         raise ValueError("either status or operation_hours is required")
    #     return values



from datetime import date, time
from enum import Enum
from pydantic import BaseModel
from typing import Optional

from . import qpu


class DayOfWeek(str, Enum):
    MONDAY = "MONDAY"
    TUESDAY = "TUESDAY"
    WEDNESDAY = "WEDNESDAY"
    THURSDAY = "THURSDAY"
    FRIDAY = "FRIDAY"
    SATURDAY = "SATURDAY"
    SUNDAY = "SUNDAY"

    def __str__(self):
        return str(self.value)


class AccessType(str, Enum):
    GROUP = "GROUP"
    ROLE = "ROLE"
    ORG = "ORG"
    QPU = "QPU"

    def __str__(self):
        return str(self.value)


class AccessSlot(BaseModel):
    day: DayOfWeek
    start_date: date
    end_date: Optional[date] = None
    start_time: time
    end_time: time


class Access(BaseModel):
    qpu_name: qpu.QPUName
    access_name: Optional[str] = None
    access_type: AccessType
    access_slots: list[AccessSlot] = []



from datetime import datetime
from typing import Union

from pydantic import field_validator, ConfigDict, BaseModel


class MessageBase(BaseModel):
    location: int = 1
    message: str
    start_datetime: datetime
    end_datetime: datetime
    model_config = ConfigDict(from_attributes=True, validate_assignment=True)


class Message(MessageBase):
    id: int


class MessageResponse(Message):
    location: int = 1
    message: str
    start_datetime: Union[str, datetime]
    end_datetime: Union[str, datetime]

    @field_validator("start_datetime", mode="before")
    @classmethod
    def formate_start_datetime(cls, value):
        return value.strftime("%m/%d/%Y @ %H:%M")

    @field_validator("end_datetime", mode="before")
    @classmethod
    def formate_end_datetime(cls, value):
        return value.strftime("%m/%d/%Y @ %H:%M")
